#ifndef base_exception_h
#define base_exception_h

// Copyright (C) Tom Horsley, 1997. All rights reserved.

#include <exception>

// A BaseException defines some additional member functions over and above
// the standard exception class. It is possible for a BaseException to
// contain a stack of reasons that describe all the calamities which
// occurred at each stage of a cascade of exceptions :-).
//
// [0] gives the last exception description pushed on the stack.
// [count()-1] gives the original exception that triggered the problem
// (or the oldest exception saved in the stack anyway).
//
class BaseException : public std::exception {
public:

   BaseException() throw();

   BaseException(const BaseException& rhs) throw();

   BaseException&
   operator=(const BaseException& rhs) throw();

   virtual 
   ~BaseException() throw();

   virtual int
   count() const throw();

   virtual const char *
   operator [] (int index)  const throw();
};

#endif // base_exception_h
