
// Copyright (C) Tom Horsley, 1997. All rights reserved.

#include <winsock2.h>
#include <windows.h>
#include "lasterr-exception.h"

LastErrorException::LastErrorException(
   const char * init_api_name,
   DWORD init_error_number,
   const char * init_fmt) throw()
: formatted(false),
  api_name(init_api_name),
  fmt(init_fmt),
  error_number(init_error_number)
{
   return;
}

LastErrorException::LastErrorException(
   const char * init_api_name,
   const char * init_fmt) throw()
: formatted(false),
  api_name(init_api_name),
  fmt(init_fmt),
  error_number(GetLastError())
{
   return;
}

LastErrorException::LastErrorException(const LastErrorException& rhs) throw()
: formatted(false),
  api_name(rhs.api_name),
  fmt(rhs.fmt),
  error_number(rhs.error_number)
{
   return;
}

LastErrorException::~LastErrorException() throw()
{
   return;
}

LastErrorException &
LastErrorException::operator=(const LastErrorException & rhs) throw()
{
   BaseException::operator=(rhs);
   formatted = false;
   api_name = rhs.api_name;
   fmt = rhs.fmt;
   error_number = rhs.error_number;
   return *this;
}

const char *
LastErrorException::what() const throw()
{
   if (! formatted) {
      char * dp = &buf[0];
      int avail = sizeof(buf) - 1;
      int len;
      const char * sp = fmt;
      char temp[50];
      while ((avail > 0) && (*sp != '\0')) {
         char c = *sp++;
         if (c == '%') {
            c = *sp++;
            switch(c) {
            case 'a':
               // PUT CODE HERE
               len = strlen(api_name);
               if (len > avail) {
                  len = avail;
               }
               strncpy(dp, api_name, len);
               dp += len;
               avail -= len;
               break;
            case 'r':
               len = CopyErrnumMsg(error_number, dp, avail);
               avail -= len;
               dp += len;
               break;
            case 'd':
               len = wsprintf(temp, "%ld", error_number);
               if (len > avail) {
                  len = avail;
               }
               strncpy(dp, temp, len);
               dp += len;
               avail -= len;
               break;
            case 'x':
               len = wsprintf(temp, "%lx", error_number);
               if (len > avail) {
                  len = avail;
               }
               strncpy(dp, temp, len);
               dp += len;
               avail -= len;
               break;
            default:
               *dp++ = c;
               --avail;
               break;
            }
         } else {
            *dp++ = c;
            --avail;
         }
      }
      *dp = '\0';
      formatted = true;
   }
   return &buf[0];
}

// Sigh... Why Microsoft goes to all the trouble to define FormatMessage and
// then leaves out all the winsock messages is beyond me...
//
// This table allows me to lookup messages for winsock errors (this was
// taken from the list of errors actually generated by winsock services as
// given in the MSDN documentation on networking).
//
const LastErrorException::extra_errmsg_info
LastErrorException::winsock_errors[] = {
   {WSAEACCES, "Permission denied."},
   {WSAEADDRINUSE, "Address already in use."},
   {WSAEADDRNOTAVAIL, "Cannot assign requested address."},
   {WSAEAFNOSUPPORT, "Address family not supported by protocol family."},
   {WSAEALREADY, "Operation already in progress."},
   {WSAECONNABORTED, "Software caused connection abort."},
   {WSAECONNREFUSED, "Connection refused."},
   {WSAECONNRESET, "Connection reset by peer."},
   {WSAEDESTADDRREQ, "Destination address required."},
   {WSAEFAULT, "Bad address."},
   {WSAEHOSTDOWN, "Host is down."},
   {WSAEHOSTUNREACH, "No route to host."},
   {WSAEINPROGRESS, "Operation now in progress."},
   {WSAEINTR, "Interrupted function call."},
   {WSAEINVAL, "Invalid argument."},
   {WSAEISCONN, "Socket is already connected."},
   {WSAEMFILE, "Too many open files."},
   {WSAEMSGSIZE, "Message too long."},
   {WSAENETDOWN, "Network is down."},
   {WSAENETRESET, "Network dropped connection on reset."},
   {WSAENETUNREACH, "Network is unreachable."},
   {WSAENOBUFS, "No buffer space available."},
   {WSAENOPROTOOPT, "Bad protocol option."},
   {WSAENOTCONN, "Socket is not connected."},
   {WSAENOTSOCK, "Socket operation on non-socket."},
   {WSAEOPNOTSUPP, "Operation not supported."},
   {WSAEPFNOSUPPORT, "Protocol family not supported."},
   {WSAEPROCLIM, "Too many processes."},
   {WSAEPROTONOSUPPORT, "Protocol not supported."},
   {WSAEPROTOTYPE, "Protocol wrong type for socket."},
   {WSAESHUTDOWN, "Cannot send after socket shutdown."},
   {WSAESOCKTNOSUPPORT, "Socket type not supported."},
   {WSAETIMEDOUT, "Connection timed out."},
   {WSAEWOULDBLOCK, "Resource temporarily unavailable."},
   {WSAHOST_NOT_FOUND, "Host not found."},
   {WSA_INVALID_HANDLE, "Specified event object handle is invalid."},
   {WSA_INVALID_PARAMETER, "One or more parameters are invalid."},
   {WSA_IO_PENDING, "Overlapped operations will complete later."},
   {WSA_IO_INCOMPLETE, "Overlapped I/O event object not in signaled state."},
   {WSA_NOT_ENOUGH_MEMORY, "Insufficient memory available."},
   {WSANOTINITIALISED, "Successful WSAStartup not yet performed."},
   {WSANO_DATA, "Valid name, no data record of requested type."},
   {WSANO_RECOVERY, "This is a non-recoverable error."},
   {WSASYSCALLFAILURE, "System call failure."},
   {WSASYSNOTREADY, "Network subsystem is unavailable."},
   {WSATRY_AGAIN, "Non-authoritative host not found."},
   {WSAVERNOTSUPPORTED, "WINSOCK.DLL version out of range."},
   {WSAEDISCON, "Graceful shutdown in progress."},
   {WSA_OPERATION_ABORTED, "Overlapped operation aborted."},
   {0, NULL}
};

int
LastErrorException::CopyErrnumMsg(
   DWORD errnum,
   char * dest,
   int destlen) throw()
{
   char msgbuf[1024];
   const char * msgptr = &msgbuf[0];
   DWORD size = FormatMessage(
                   FORMAT_MESSAGE_FROM_SYSTEM,
                   NULL,
                   errnum,
                   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                   msgbuf,
                   sizeof(msgbuf),
                   NULL);
   if (size == 0) {
      // Well, FormatMessage doesn't seem to know 'nuthin, lets see if
      // it is one of the messages in our table...
      for (int i = 0; winsock_errors[i].errmsg != NULL; ++i) {
         if (winsock_errors[i].errnum == errnum) {
            msgptr = winsock_errors[i].errmsg;
            size = strlen(msgptr);
         }
      }
   } else {
      // I want to put these messages inside other strings, but the
      // lovely message I actually get back has CRLF crap in it.
      // Change them all to spaces...
      for (int i = 0; i < size; ++i) {
         char c = msgbuf[i];
         if ((c == '\r') || (c == '\n')) {
            msgbuf[i] = ' ';
         }
      }
      while ((size >= 0) && (msgbuf[size-1] == ' ')) {
         --size;
      }
   }
   if (size == 0) {
      // Nobody knows 'nuthin about this error code, just report that...
      size = wsprintf(msgbuf, "Unknown error code 0x%08lx", errnum);
   }
   if (size > destlen) {
      size = destlen;
   }
   strncpy(dest, msgptr, size);
   return size;
}
