#ifndef lasterr_exception_h
#define lasterr_exception_h

// Copyright (C) Tom Horsley, 1997. All rights reserved.

#include "base-exception.h"
#include <windows.h>

// LastErrorException is used to record exception info generated because
// of a failure in a win32 API which sets the GetLastError() value.
//
// The format string says how to format the error message when requested.
// It supports the following format codes:
//
// %a - replace with the name of the win32 function (passed in as api_name).
// %r - replace with the human readable error message from FormatMessage
//      (or a substitute giving the error code if FormatMessage has nothing
//      useful to say).
// %d - replace with the decimal format value of the GetLastError() code.
// %x - replace with the hex format value of the GetLastError() code.
// %% - replace with a single % character.
//
// Any other info in format string is simply copied literally into the
// message, so it may contain additional information (like why you were
// trying to call the given function). The default format string generates
// an error much like the perror() function.
//
class LastErrorException : public BaseException {
public:

   LastErrorException(
      const char * init_api_name,
      DWORD init_error_number,
      const char * init_fmt = "%a: %r") throw();

   LastErrorException(
      const char * init_api_name,
      const char * init_fmt = "%a: %r") throw();

   LastErrorException(const LastErrorException& rhs) throw();

   virtual
   ~LastErrorException() throw();

   LastErrorException &
   operator=(const LastErrorException & rhs) throw();

   virtual const char *
   what() const throw();

private:

   mutable char buf[256];
   mutable bool formatted;
   const char * api_name;
   const char * fmt;
   DWORD error_number;

   struct extra_errmsg_info {
      DWORD errnum;
      const char * errmsg;
   };
   static const extra_errmsg_info winsock_errors[];

      // Copy the explainatory message for the given error code into the
      // destination buffer. Copy no more than 'destlen' characters.  return
      // number of characters actually copied.
      //
   static int
   CopyErrnumMsg(DWORD errnum, char * dest, int destlen) throw();
};

#endif // lasterr_exception_h
