
// Copyright (C) Tom Horsley, 1997. All rights reserved.

#include "logging.h"
#include <exception>
#include <stdarg.h>

LogStuff::LogStuff()
{
   return;
}

LogStuff::~LogStuff()
{
   return;
}

void
LogStuff::LogException(exception & /* ee */)
{
   return;
}

void
LogStuff::LogString(const char * /* str */)
{
   return;
}

void
LogStuff::LogPrintf(const char * /* fmt */, ...)
{
   return;
}

LogDirectory::LogDirectory(const char * init_dirname, const char * init_suffix)
: dirname(init_dirname), suffix(init_suffix),
  curlog_name(NULL), curlog(NULL)
{
   return;
}

LogDirectory::~LogDirectory()
{
   delete [] curlog_name;
}

void
LogDirectory::LogException(exception & ee)
{
   Lock();
   if (curlog != NULL) {
      const char * msg = ee.what();
      if (msg != NULL) {
         InternLogString(msg);
      } else {
         InternLogString("unidentified exception");
      }
   }
   UnLock();
}

void
LogDirectory::LogString(const char * str)
{
   Lock();
   if (curlog != NULL) {
      InternLogString(str);
   }
   UnLock();
}

void
LogDirectory::LogPrintf(const char * fmt, ...)
{
   Lock();
   if (curlog != NULL) {
      char buf[1024];
      va_list args;
      va_start(args, fmt);
      wvsprintf(buf, fmt, args);
      va_end(args);
      InternLogString(buf);
   }
   UnLock();
}

void
LogDirectory::InternLogString(const char * str)
{
   char buf[1024];

   wsprintf(buf, "%04d/%02d/%02d %02d:%02d:%02d UTC [%d/%d]: ",
      now.wYear, now.wMonth, now.wDay, now.wHour, now.wMinute, now.wSecond,
         GetCurrentProcessId(), GetCurrentThreadId());
   fputs(buf, curlog);
   fputs(str, curlog);
   if (str[strlen(str)-1] != '\n') {
      fputs("\n", curlog);
   }
}

void
LogDirectory::Lock()
{
   EnterCriticalSection(log_lock);
   GetSystemTime(&now);
   char yymmdd[7];
   sprintf(yymmdd, "%02d%02d%02d", now.wYear % 100, now.wMonth, now.wDay);
   char * new_log_name = new char [strlen(dirname) +    // directory
                                   1 +                  // backslash
                                   6 +                  // yymmdd
                                   1 +                  // dot
                                   strlen(suffix) +     // suffix
                                   1];                  // null terminator
   strcpy(new_log_name, dirname);
   strcat(new_log_name, "\\");
   strcat(new_log_name, yymmdd);
   strcat(new_log_name, ".");
   strcat(new_log_name, suffix);
   if ((curlog_name == NULL) || (strcmp(curlog_name, new_log_name) != 0)) {
      delete [] curlog_name;
      curlog_name = new_log_name;
      if (curlog != NULL) {
         fclose(curlog);
      }
      curlog = fopen(curlog_name, "a+");
   } else {
      delete [] new_log_name;
   }
}

void
LogDirectory::UnLock()
{
   if (curlog != NULL) {
      fflush(curlog);
   }
   LeaveCriticalSection(log_lock);
}
