#ifndef logging_h
#define logging_h

// Copyright (C) Tom Horsley, 1997. All rights reserved.

#include <stdio.h>
#include <windows.h>
#include "wrap-critical.h"

class exception;

// Base class does nothing, but it is convenient to make one of these in
// order to not have to check for NULL pointer all the time when logging
// information.
//
class LogStuff {
public:

   LogStuff();

   virtual
   ~LogStuff();

   virtual void
   LogException(exception & ee);

   virtual void
   LogString(const char * str);

   virtual void
   LogPrintf(const char * fmt, ...);
};

// A class to manage logging entries to a family of log files in a
// given directory. (You get a new log files every day).
//
class LogDirectory : public LogStuff {
public:

      // Constructor takes directory name and suffix string for
      // log files.
      //
   LogDirectory(const char * init_dirname, const char * init_suffix);

   ~LogDirectory();

   virtual void
   LogException(exception & ee);

   virtual void
   LogString(const char * str);

   virtual void
   LogPrintf(const char * fmt, ...);

private:

   void
   InternLogString(const char * str);

   void
   Lock();

   void
   UnLock();

   DeclareCriticalSection log_lock;

   char * curlog_name;

   FILE * curlog;

   const char * dirname;
   const char * suffix;

   SYSTEMTIME now;
};

#endif // logging_h
