#ifndef ntp_packet_h
#define ntp_packet_h

// Copyright (C) Tom Horsley, 1997. All rights reserved.

#include "ntp-long-fixed.h"

// This is the format of the basic NTP network packet (as documented
// in rfc1769). The SNTP algorithms implemented here ignore lots
// of these fields. The basic packet is the only thing this program
// sends.
//
struct BasicNtpPacket {
   unsigned char          li_vn_mode;                   // packed li,vn, & mode
   unsigned char          stratum;                      // 1-15
   unsigned char          poll;                         // log2(seconds)
   signed   char          precision;                    // log2(clockfreq)
   long                   root_delay;                   // 32 bit fixed point
   long                   root_dispersion;              // 32 bit fixed point
   char                   reference_identifier[4];      // clock name
   NetworkOrderLongFixed  reference_timestamp;          // 64 bit fixed point
   NetworkOrderLongFixed  originate_timestamp;          // 64 bit fixed point
   NetworkOrderLongFixed  receive_timestamp;            // 64 bit fixed point
   NetworkOrderLongFixed  transmit_timestamp;           // 64 bit fixed point
};

// The authentication info is optional, and is also ignored in this SNTP code.
//
struct OptionalAuthenticationInfo {
   unsigned long key_id;
   union {
   unsigned char des_auth[8];
   unsigned char md5_auth[16];
   };
};

// This is the largest possible packet I might see.
//
struct FullNtpPacket {
   BasicNtpPacket              basic_part;
   OptionalAuthenticationInfo  auth_part;
};

// Helper functions for packing/unpacking the li,vn.mode fields in the
// first byte of the packet.

inline unsigned char
PacketLeapIndicator(BasicNtpPacket & p)
{
   return (unsigned char)((p.li_vn_mode >> 6) & 0x3);
}

inline unsigned char
PacketVersion(BasicNtpPacket & p)
{
   return (unsigned char)((p.li_vn_mode >> 3) & 0x7);
}

inline unsigned char
PacketMode(BasicNtpPacket & p)
{
   return (unsigned char)(p.li_vn_mode & 0x7);
}

inline void
SetPacketLeapVersionMode(
   BasicNtpPacket & p,
   unsigned char new_li,
   unsigned char new_vn,
   unsigned char new_mode)
{
   p.li_vn_mode = (unsigned char)(((new_li & 0x3) << 6) |
                                  ((new_vn & 0x7) << 3) |
                                  (new_mode & 0x7));
}

#endif // ntp_packet_h
