
// Copyright (C) Tom Horsley, 1997. All rights reserved.

#include <string.h>
#include <ctype.h>
#include "ntp-service.h"
#include "lasterr-exception.h"
#include "logging.h"

int WINAPI
WinMain(
   HINSTANCE hInstance,
   HINSTANCE hPrevInstance,
   LPSTR lpCmdLine,
   int nCmdShow)
{
   NtpService ns(hInstance);
   try {
      LPSTR cmdarg = NULL;
      if (lpCmdLine != NULL) {
         while (isspace(*lpCmdLine)) ++lpCmdLine;
         cmdarg = lpCmdLine;
         while ((*lpCmdLine != '\0') && (! isspace(*lpCmdLine))) ++lpCmdLine;
         *lpCmdLine = '\0';
      }
      if ((cmdarg != NULL) && (cmdarg[0] != '\0')) {
         char * msgbuf = new char [4096];
         if (cmdarg[0] == '/') {
            cmdarg[0] = '-';
         }
         if (cmdarg[1] == '/') {
            cmdarg[1] = '-';
         }
         if ((strcmp(cmdarg, "--help") == 0) ||
             (strcmp(cmdarg, "-h") == 0) ||
             (strcmp(cmdarg, "-H") == 0) ||
             (strcmp(cmdarg, "-?") == 0)) {
            strcpy(msgbuf,
"ntptime - win32 simple NTP time setting service\n\
usage: ntptime [--help] [--install] [--uninstall]\n\
\n\
--help          print this message and exit\n\
--install       install this as a service on windows NT\n\
--uninstall     remove this as a windows NT service\n\
--version       print version and copyright information\n\
\n\
ntptime runs as a service on NT and a normal process on Windows 95.  It sets\n\
the system time periodically from a network NTP server. It can be configured\n\
to wait for a RAS connection before trying to set the time, set the time\n\
only once, or set the time periodically as long as the network connection is\n\
active.\n");
            MessageBox(NULL, msgbuf, "NTPTime", MB_OK);
            return 0;
         } else if ((strcmp(cmdarg, "--version") == 0) ||
                    (strcmp(cmdarg, "-v") == 0) ||
                    (strcmp(cmdarg, "-V") == 0)) {
            strcpy(msgbuf, "ntptime version ");
            strcat(msgbuf, NtpService::ntpversion);
            strcat(msgbuf, "\nCopyright (C) Tom Horsley 1997.\n\
Unlimited distribution and use of unaltered program allowed.\n\
\n\
Tom.Horsley@worldnet.att.net\n");
            MessageBox(NULL, msgbuf, "NTPTime", MB_OK);
            return 0;
         } else if ((strcmp(cmdarg, "--install") == 0) ||
                    (strcmp(cmdarg, "-i") == 0) ||
                    (strcmp(cmdarg, "-I") == 0)) {
            if (ns.IsRunningOnNT()) {
               if (ns.Install(SERVICE_AUTO_START)) {
                  strcpy(msgbuf, "NTP service successfully installed\n");
                  MessageBox(NULL, msgbuf, "NTPTime", MB_OK);
                  return 0;
               } else {
                  strcpy(msgbuf,
                         "Unable to install service - check privileges\n");
                  MessageBox(NULL, msgbuf, "NTPTime",
                             MB_OK | MB_ICONEXCLAMATION);
                  return 2;
               }
            } else {
               fputs("--install not required on Windows 95\n", stderr);
               return 2;
            }
         } else if ((strcmp(cmdarg, "--uninstall") == 0) ||
                    (strcmp(cmdarg, "-u") == 0) ||
                    (strcmp(cmdarg, "-U") == 0)) {
            if (ns.IsRunningOnNT()) {
               if (ns.Uninstall()) {
                  strcpy(msgbuf, "NTP service successfully removed\n");
                  MessageBox(NULL, msgbuf, "NTPTime", MB_OK);
                  return 0;
               } else {
                  strcpy(msgbuf,
                         "Unable to remove service - check privileges\n");
                  MessageBox(NULL, msgbuf, "NTPTime",
                             MB_OK | MB_ICONEXCLAMATION);
                  return 2;
               }
            } else {
               strcpy(msgbuf, "--uninstall not required on Windows 95\n");
               MessageBox(NULL, msgbuf, "NTPTime",
                          MB_OK | MB_ICONEXCLAMATION);
               return 2;
            }
         } else {
            strcpy(msgbuf, "Unrecognized argument: ");
            strcat(msgbuf, cmdarg);
            MessageBox(NULL, msgbuf, "NTPTime",
                       MB_OK | MB_ICONEXCLAMATION);
            return 2;
         }
      } else {
         if (! ns.StartService()) {
            return 2;
         }
      }
   } catch (exception & ee) {
      ns.GetLog()->LogException(ee);
      return 2;
   }
   return 0;
}
